class WuSView
{
  WuSGameState gameState;
  PImage gameBox;
  PImage squareBlack, squareWhite;
  PImage lampWhite, lampRed, lampWhiteOn, lampRedOn;
  PImage  connectorJack;
  //where are lamps installed?
  private int[][] lampPositions = { // 0: no lamp, 1: sheep lamp, 2: sheep and wolf, 3: wolf only
    {
      0, 1, 0, 1, 0, 1, 0, 1
    }
    , 
    {
      1, 0, 2, 0, 1, 0, 1, 0
    }
    , 
    {
      0, 1, 0, 2, 0, 2, 0, 2
    }
    , 
    {
      2, 0, 2, 0, 2, 0, 2, 0
    }
    , 
    {
      0, 2, 0, 2, 0, 2, 0, 2
    }
    , 
    {
      3, 0, 2, 0, 2, 0, 2, 0
    }
    , 
    {
      0, 2, 0, 3, 0, 3, 0, 3
    }
  };
  //which lamps are on?
  private int[][] lampStatus = { // 0: all off, 1: sheep on, 2: sheep and wolf on, 3: wolf on
    {
      0, 1, 0, 1, 0, 1, 0, 1
    }
    , 
    {
      0, 0, 0, 0, 0, 0, 0, 0
    }
    , 
    {
      0, 0, 0, 0, 0, 0, 0, 0
    }
    , 
    {
      0, 0, 0, 0, 0, 0, 0, 0
    }
    , 
    {
      0, 0, 0, 0, 0, 0, 0, 0
    }
    , 
    {
      0, 0, 0, 0, 0, 0, 0, 0
    }
    , 
    {
      0, 3, 0, 0, 0, 0, 0, 0
    }
  };
  //where are the jacks? The positions refer to the "logical" location. 
  //The actual connectors are below the field
  private int[][] jackPositions = { // 0: no jack, 1: jack
    {
      0, 0, 0, 0, 0, 0, 0, 0
    }
    , 
    {
      0, 0, 0, 0, 0, 0, 0, 0
    }
    , 
    {
      0, 0, 0, 1, 0, 1, 0, 1
    }
    , 
    {
      1, 0, 1, 0, 1, 0, 1, 0
    }
    , 
    {
      0, 1, 0, 1, 0, 1, 0, 1
    }
    , 
    {
      1, 0, 1, 0, 1, 0, 1, 0
    }
    , 
    {
      0, 1, 0, 1, 0, 1, 0, 1
    }
  };

  public int size, width, height; //width and height of game field

  public WuSView(WuSGameState gameState, int size)
  {
    this.gameState = gameState;
    this.size=size;
    this.width = (int)(size*0.75);
    this.height = (int)(size*0.65);
    //load textures
    gameBox = loadImage("GameBox.png");
    squareBlack = loadImage("square_black.png");
    squareWhite = loadImage("square_white.png");
    lampWhite = loadImage("lamp_white.png");
    lampWhiteOn = loadImage("lamp_white_on.png");
    lampRed = loadImage("lamp_red.png");
    lampRedOn = loadImage("lamp_red_on.png");
    connectorJack = loadImage("connector-jack.png");
  } 

  /**
   function draw()
   Draws the game box, the game field and the connectors
   **/
  public void draw()
  {
    imageMode(CORNER);
    image(gameBox, 0, 0, size, size*1.246);
    pushMatrix();
    translate(size*0.13, size*0.4);
    drawGameField();
    popMatrix();
    drawCable();
  }

  /**
   function drawGameField()
   Draws all squares, lamps and connectors of the gameField. Depending on the status of the lamps, use on/off texture
   **/
  public void drawGameField()
  {
    noStroke();
    //////////////////////////////
    //first draw the chess board, in the Gräff version there are only 7 rows
    imageMode(CORNER); //positions are upperl eft corner of square
    for (int row=0; row < 7;row++)
    {
      for (int column=0; column < 8; column++)
      {
        if ((row+column)%2==0)
          image(squareBlack, column*width/8, row*height/7, width/8, height/7);
        else
          image(squareWhite, column*width/8, row*height/7, width/8, height/7);
      }
    }

    //get the state in array form
    int[][] sheepPositions = gameState.getSheepPositionsArray();
    int[] wolfPosition = gameState.getWolfPositionArray();
    for (int row=0; row < 7;row++)
    {
      for (int column=0; column < 8; column++)
      {
        //////////////////////////////
        //Now the lamps
        imageMode(CENTER); //positions are center of lamp, not corner
        //white lamps
        if (lampPositions[row][column]==1 || lampPositions[row][column]==2)
        {
          if (sheepPositions[row][column]==1) //on
            image(lampWhiteOn, column*width/8+(width/8)/2, row*height/7+height/7-height*0.035, width*0.045, height*0.045);
          else //off
          image(lampWhite, column*width/8+(width/8)/2, row*height/7+height/7-height*0.035, width*0.045, height*0.045);
        }
        //red lamps
        if (lampPositions[row][column]==2 || lampPositions[row][column]==3)
        {
          if (wolfPosition[0]==row && wolfPosition[1]==column) //on
              image(lampRedOn, column*width/8+(width/8)/2, row*height/7+height*0.04, width*0.045, height*0.045);
          else //off
          image(lampRed, column*width/8+(width/8)/2, row*height/7+height*0.04, width*0.045, height*0.045);
        }
        //////////////////////////////
        //Finally the connector jacks
        if (jackPositions[row][column]==1)
        {
          image(connectorJack, ((float)column)/8*width+(width/32), ((float)row+1)/7*height + height*0.015, width*0.02, height*0.02);
          image(connectorJack, ((float)column)/8*width+(width/32*3), ((float)row+1)/7*height + height*0.015, width*0.02, height*0.02);
        }
        //////////////////////////////
        // If the wolf lost, we display a message
        if (this.gameState.wolfLost == true)
        {
          fill(255, 0, 0);
          textSize(70);
          text("You lost!", width/2-100, height/2);
        }
        //////////////////////////////
        //Debug: Show Field Names
        if (DEV_MODE)
        {
          fill(0, 102, 153);
          textSize(12);
          if ((row+column)%2==0) //white field
            text(
            Character.toString((char)(65+(((row%2==0) ? 7 : 8) - column)/2))+ //Column (int to ascii with 65(=A) offset)
            (row+1), //row
            (column+((row%2==0) ? 1 : -1))*width/8, //add/substract 1 if in odd/even row
            (0.5+row)*height/7);
        }
      }
    }
  }
  public void drawCable()
  {
    noFill();
    strokeWeight(5);
    stroke(220, 200, 100);
    bezier(size*0.22, size*1.21, 
    size*0.22+50, size*1.21-20, 
    (mouseX-size*0.22)/2, (mouseY+size*1.21)/1.5, 
    mouseX, mouseY);
  }
}

